/*   Midi6 - C++ Implementation of MPU401 Midi !! */

//	(c) 1990 BP Midi Systems

#include<stdio.h>
#include<dos.h>

#include"midi6.h"

MIDI midi(256);

#pragma -N-

void (*midi_play)(unsigned char c)=NULL;

void mreset(void)
{
	// this gets executed no matter what at exit to insure MPU is empty!
	outportb(0x331,0xff);		// reset MPU
	delay(200);
	for (int i=0;i<32767;i++)
	inportb(0x330);
}

#pragma exit mreset 100


void interrupt MidiInterruptHandler(...)
{
	unsigned i;
	// must compile this function with stack checking off!
	disable();
	if (!(DSR&inportb(MIDI_STATUS_PORT))) {

		i=inportb(MIDI_DATA_PORT);
		if (i>=0xf0 && i<=0xf7) {
			if (midi_play) midi_play(i&0x0f);
		}
		else
			midi.put(i);

	  //	pokeb(0xb800,159,(peekb(0xb800,159)+1));     /* flashing lights */
		outportb(PIC_A,EOI);			      /* signal End of Interrupt */
	}
	enable();
}

void _ena8259(int int_no)
{
    unsigned t;
    t=inportb(PIC_B);
    t=t & ~ (1 << int_no);
    outportb(PIC_B,t);
 }

void _dis8259(int int_no)
{
    unsigned t;
    t=inportb(PIC_B);
    t=t | (1 << int_no);
    outportb(PIC_B,t);
}

MIDI::MIDI(int size): ByteQ(size)
{
	/* install the C midi interrupt handler */
	old_int=getvect(8+MIDI_INTERRUPT);
	setvect(8+MIDI_INTERRUPT,MidiInterruptHandler);
	/* enable NMI on 8259 PIC */
	_ena8259(MIDI_INTERRUPT);

	Open=TRUE;
}

MIDI::~MIDI(void)
{
	if (Open) {
		_dis8259(MIDI_INTERRUPT);
		setvect(8+MIDI_INTERRUPT,old_int);	    /* restore the old guy */
		Open=FALSE;
	}
}

MIDI& MIDI::operator << (byte b)
{
	while (DRR & inportb(MIDI_STATUS_PORT));     /* wait till recv ready */
	outportb(MIDI_DATA_PORT,b);
	return *this;
}

void MIDI::Write(byte *b, int len)
{
	while (DRR & inportb(MIDI_STATUS_PORT));     /* wait till recv ready */
	for (register i=0;i<len;i++)
		outportb(MIDI_DATA_PORT,b[i]);
}

void MIDI::wait(void)
{
	// wait till data stops coming in
	delay(500);
	int l=0;
	for (int i=1;i<60;i++) {
		if (midi.isQd==l) break;
		l=midi.isQd;
		delay(500);
	}
}

BOOL MIDI::xfer(byte far *b,int l)
{
	if (isQd<l) {
		printf("xfer called, not enough data!\a\n");
		return FALSE;
	}
	for (int i=0;i<l;i++) b[i]=get();
	return TRUE;
}

BOOL MIDI::operator () (byte cmd)
{
	while (DRR & inportb(MIDI_STATUS_PORT));     /* wait till recv ready */
	outportb(MIDI_CMD_PORT,cmd);

	while (!midi.isQd);
	if (midi.get()!=ACK) return FALSE;
		else return TRUE;
}

/*----- my nice and simple Q implementation -----*/

ByteQ::ByteQ(int size)
{
	qstart = new byte [size];
	qend = qstart+size;
	qhead=qtail=qstart;
	isQd=0;

}

void ByteQ::flush(void)
{
	qhead=qtail=qstart;
	isQd=0;
}

ByteQ::~ByteQ(void)
{
	delete qstart;
}

void ByteQ::put(byte b)
{
	*qhead=b;
	isQd++;
	if (++qhead==qend) qhead=qstart;
}

byte ByteQ::get(void)
{
	if (isQd) {
		byte b=*qtail;
		if (++qtail==qend) qtail=qstart;
		isQd--;
		return b;
	} else return ACK;		// nothing to return so return only safe value
}



