/*[]-------------------------- Gadgets ----------------------------[]

  Implements Gadget routines in C++

  []---------------------------------------------------------------[]*/

#include"ultimate.h"

void (*gadg_idle)()=NULL;
BOOL (*gadg_kbd)(unsigned int x)=NULL;

Gadget::Gadget(int x1, int y1, struct GadgAttr *g, char *T):
			x(x1), y(y1), Title(T),
			Attr(g) { delta=NULL; }

GadgetDialog::GadgetDialog(int x1, int y1,
		int x2, int y2,	int attr,
		char *T,
		GadgetGroup **g, int gc):

		Title(T),
		GroupList(g),
		NumberOfGroups(gc),
		Wi(x1,y1,x2,y2,attr)
{
	CurrentGroup=0;
}

void IntGadget::Display(void)
{
	char wk[25];
	gotoxy(x+strlen(Title)+1,y);
	textattr(Attr->b);
	sprintf(wk,"%03d",Value);
	cputs(wk);			// write it!
}

void ListGadget::Display(void)
{
	char wk[25];
	gotoxy(x+strlen(Title)+1,y);
	textattr(Attr->b);
	sprintf(wk,"%-10s",list[Value]);
	cputs(wk);			// write it!
}

void HexGadget::Display(void)
{
	char wk[25];
	gotoxy(x+strlen(Title)+1,y);
	textattr(Attr->b);
	sprintf(wk,"0x%02X",Value);
	cputs(wk);			// write it!
}

void BoolGadget::Display(void)
{
	gotoxy(x+strlen(Title)+1,y);
	textattr(Attr->b);
	cputs(Value?"True ":"False");			// write it!
}

void AnalogGadget::Display(void)
{
	char wk[25];
	gotoxy(x+strlen(Title)+1,y);
	textattr(Attr->b);
	sprintf(wk,"%02d",Value);
	cputs(wk);			// write it!

	textattr( aAttr );
	gotoxy(x,y+1);
	putch(17);
	BiosWrt(177, aAttr,Width);
	gotoxy(x+1+Width,y+1);
	putch(16);
	int offs=(float)Value/((float)((Maximum-Minimum+1)/Width)+.7);

	gotoxy(x+offs+1,y+1);
	putch(254);
}

struct HotKey {
	char c;
	int gg,g;
	};

int HotKeyCount=0;
HotKey HotKeys[20];

// This table refrences uppercase chars to their extended Alt Equiv
// as returned by getch()
int HotKeyEquiv[100]=
{	00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,    //000-019
	00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,    //020-039
	00,00,00,00,00,00,00,00,00,120,121,122,123,124,125,126,127,128,129,00,    //040-059
	00,00,00,00,00,30,48,46,32,18,33,34,35,23,36,37,38,50,49,24,    //060-079
	25,16,19,31,20,22,47,17,45,21,44,00,00,00,00,00,00,00,00,00,    //080-099
	};

void HotKeyAdd(char c,int gg, int g)  // gadgetgroup number & gadget number
{
// pass me any char - I convert to uppercase and find its extended
// alt equiv from table
	HotKeys[HotKeyCount].c=HotKeyEquiv[toupper(c)];
	HotKeys[HotKeyCount].g=g;
	HotKeys[HotKeyCount++].gg=gg;
}

void ButtonGadget::DisplayTitle(void)
{
	Gadget::DisplayTitle();

	textattr(Attr->b);
	gotoxy(x-1,y);
	putch('');
	gotoxy(x-1,y+1);
	BiosWrt('',Attr->b,strlen(Title)-1);
}


void Gadget::DisplayTitle(void)
{
	gotoxy(x,y);
	for (int i=0;i<strlen(Title);i++) {
		if (Title[i]!='~') {
			textattr(Attr->a);
			putch(Title[i]);
		}
		else {
			textattr(Attr->d);
			putch(Title[++i]);
		}
	}
}

void Gadget::open(void)
{
	// display a gadget
	DisplayTitle();
	Display();
}

void GadgetGroup::open(void)
{
	gotoxy(x,y);
	textattr(GroupAttr);
	cputs(Title);
	// display gadgets
	for (int i=0;i<NumberOfGadgets;i++)
		GadgetList[i]->open();
}

GadgetGroup::GadgetGroup(int x1, int y1, int gra, char *T,
			Gadget **g,int c):
			x(x1), y(y1), GroupAttr(gra),
			Title(T),
			GadgetList(g), NumberOfGadgets(c)
{
	CurrentGadget=0;
}

void BoolGadget::inc(void)
{
	(*this)<< (!Value);
}

void IntGadget::inc(void)
{
	(*this)<< (Value+1);
}

void IntGadget::dec(void)
{
	(*this)<< (Value-1);
}

void IntGadget::operator << (int i)
{
	if ( (i>=Minimum) && (i<=Maximum))
		Value=i;
	Display();
}

void swap(int &a, int &b)
{
	int c=b; b=a; a=c;
}

void GadgetDialog::open(void)
{
	int i,j,k;
	this->brdr=SINGLE;
	this->Wi::open(Title);
	// display groups
	for (i=0;i<NumberOfGroups;i++)
		GroupList[i]->open();

	// add hotkeys
	HotKeyCount=0;
	for (i=0;i<NumberOfGroups;i++)
		for (j=0;j<GroupList[i]->NumberOfGadgets;j++)
			for (k=0;k<strlen(GroupList[i]->GadgetList[j]->Title);k++)
				if (GroupList[i]->GadgetList[j]->Title[k]=='~')
					HotKeyAdd(GroupList[i]->GadgetList[j]->Title[++k],i,j);
}

void GadgetDialog::exec(void)
{
	BOOL done=FALSE;
	GadgetGroup *gg;
	Gadget *g;

	while (!done) {

		// make sure this window active & top
		if (!top) act();

		// highlight current gadg
		gg=GroupList[CurrentGroup];

		g=gg->GadgetList[gg->CurrentGadget];

		// temp exchng normal & highlight attrs

		swap( g->Attr->b, g->Attr->c);
		g->Display();

		// read kybd or call idle routine if there
		while (!kbhit())
			if (gadg_idle!=NULL) {
				gadg_idle();			// call users idle routine
			}
		int c=getchx();

		// make sure this window active & top
		if (!top) act();

		// unhighlight current gadg
		swap( g->Attr->b, g->Attr->c);
		g->Display();

		switch (c) {

	// down - next gadg
			case 0x0150:
				if (++(gg->CurrentGadget) == gg->NumberOfGadgets)
					gg->CurrentGadget=0;
				break;

	// up - prev gadg
			case 0x0148:
				if (--(gg->CurrentGadget) < 0)
					gg->CurrentGadget=gg->NumberOfGadgets-1;
				break;

	// left - prev group
			case 0x014b:
				if (--CurrentGroup < 0)
					CurrentGroup=NumberOfGroups-1;
				break;
	// + - increment gadg
			case 0x2b:
				g->inc();
				if (g->delta) g->delta(g);
				break;

	// - - decrement gadg
			case 0x2d:
				g->dec();
				if (g->delta) g->delta(g);
				break;

	// right - next group
			case 0x014d:
				if (++CurrentGroup == NumberOfGroups)
					CurrentGroup=0;
				break;

	// default - pass to user, if not handled check hot keys
			default:
				if (gadg_kbd!=NULL)
					if (gadg_kbd(c)) break;
				c&=0xff;
				for (int i=0;i<HotKeyCount;i++)
					if (c==HotKeys[i].c) {
						CurrentGroup=HotKeys[i].gg;
						GroupList[CurrentGroup]->CurrentGadget=HotKeys[i].g;
					}
		}
	}
}

